//
//  NSAttributedString+BonMotUtilities.h
//  BonMot
//
//  Created by Eliot Williams on 3/9/16.
//
//

#import <Foundation/Foundation.h>
#import "BONCompatibility.h"

BON_ASSUME_NONNULL_BEGIN

@interface NSAttributedString (BonMotUtilities)

/**
 *  The equivalent of calling @c -bon_humanReadableStringIncludingImageSize: and passing @c YES.
 */
@property (copy, nonatomic, readonly) NSString *bon_humanReadableString;

/**
 *  A string representing the attributed string, but with
 *  human-readable substitutes for all characters that are hard
 *  to read in a monospace font, such as non-printing characters
 *  and ambiguous punctuation. Use the string generated by this
 *  method to create easily readable and maintainable unit tests.
 *
 *  @param shouldIncludeImageSize Whether to include the size of
 *  any embedded images in the resulting string. Pass @c NO if
 *  you are writing unit tests, and your images are of non-
 *  deterministic size.
 *
 *  @return A human readable version of the attributed string’s @c string.
 */
- (NSString *)bon_humanReadableStringIncludingImageSize:(BOOL)shouldIncludeImageSize;

@end

BON_ASSUME_NONNULL_END
